#include "y_axis_driver.h"

#define Y_STEP 26  
#define Y_DIRECTION 27
#define Y_PULSE_WIDTH (CLKFREQ / 1000000)
#define Y_MIN_SPEED (CLKFREQ / 5714)

/// cog variables
static int y_axis_cog = 0;
static int y_axis_stack[100];

volatile bool y_move_complete;

void y_axis_driver(void *par);

void y_axis_init()
{
	if(y_axis_cog == 0)
	{
		y_axis_cog = 1 + cogstart(&y_axis_driver, NULL,
			y_axis_stack, sizeof y_axis_stack);
	}
}

/// stop y_axis cog
void y_axis_stop()
{
	if(y_axis_cog > 0)
	{
		cogstop(y_axis_cog - 1);
		y_axis_cog = 0;
	}
}

void y_axis_driver(void *par)
{
	int32_t y_counter;

	// Set DIRA as an output.
	DIRA |= 1 << Y_DIRECTION;

	do
	{
		if(app_move_now == true)
		{
			app_move_now = false;

			// Set up the CTRMODE of Counter A for NCO/PWM single-ended.
			CTRA = (4 << 26) | Y_STEP;

			// Set the value to be added to PHSA with every clock cycle.
			FRQA = 1;

			// Set DIRA as an output.
			DIRA |= 1 << Y_STEP;

			// Set the OUTA register to match the desired direction of rotation.
			if(app_y_dir == 0)
			{
				OUTA &= (~1 << Y_DIRECTION);
			}
			else
			{
				OUTA |= 1 << Y_DIRECTION;
			}

			// Get the current System Counter value.
			y_counter = CNT;

			while(app_y_running_steps != 0)
			{
				// Send out a high pulse on the step pin for the desired duration.
				PHSA = -Y_PULSE_WIDTH;

				// Wait for a specified period of time before sending another
				// high pulse to the step pin.
				waitcnt(y_counter += Y_MIN_SPEED);

				// Decrement running_steps
				app_y_running_steps--;
			}

			y_move_complete = true;
		}
	}
	while(1);
}